package calendar;

import java.util.Enumeration;

import javax.microedition.io.Connector;
import javax.microedition.io.file.FileConnection;
import javax.microedition.io.file.FileSystemRegistry;

import log.ILogger;

/**
 * Assists with caching calendars downloaded from the net.
 * Simply creates String filenames of file caches 
 * 
 * @author richard
 *
 */
public class CacheFileManager {

	private static CacheFileManager instance;

	private static final String CACHE_DIR_NAME = "PhoneCalendarCache/";

	private final String cacheDirURL;

	private CacheFileManager(ILogger log) {
		log.log("Creating cache", ILogger.INFO);
		final String previousCache = searchForCache(log, false);
		if (previousCache != null) {
			log.log("Found old cache " + previousCache, ILogger.INFO);
			cacheDirURL = previousCache;
		} else {
			log.log("Attempting to create new one", ILogger.INFO);
			cacheDirURL = searchForCache(log, true);
		}
		if (cacheDirURL == null)
			throw new RuntimeException("Unable to find or create cache");
	}

	public String createCacheNode(String name) {
		return cacheDirURL + name;
	}

	private String searchForCache(ILogger log, boolean createIfPossible) {
		String cacheURL = null;
		log.log("Attempting to list roots", ILogger.INFO);
		for (Enumeration roots = FileSystemRegistry.listRoots(); roots.hasMoreElements();) {
			final String root = (String) roots.nextElement();
			final String attemptedURL = "file:///" + root + "/" + CACHE_DIR_NAME;
			log.log("Checking " + attemptedURL + " will " + (createIfPossible ? "" : "not ") + "create if necessary",
					ILogger.INFO);
			try {
				final FileConnection connection = (FileConnection) Connector.open(attemptedURL);
				if (connection != null && connection.exists()) {
					log.log("Found cache dir on " + attemptedURL, ILogger.INFO);
					cacheURL = attemptedURL;
					break;
				} else if (createIfPossible) {
					log.log("Creating new cache dir on " + attemptedURL, ILogger.INFO);
					connection.mkdir();
					cacheURL = attemptedURL;
					break;
				}
			} catch (Exception e) {
				log.log("Error attempting to access root " + attemptedURL + ": " + e.getMessage(), ILogger.WARNING);
			}
		}
		return cacheURL;
	}

	public static synchronized CacheFileManager getInstance(ILogger log) {
		if (instance == null)
			instance = new CacheFileManager(log);
		return instance;
	}
}
