package calendar;

import log.ILogger;
import calendar.objectmodel.ICalendar;

/**
 * ICalendarSource implementation which used an IRawCalendarSource and an ICalendarParser
 * to create the ICalendar.
 * 
 * i.e. a FileConnectionCalendarSource (an IRawCalendarSource) might be bundled with the 
 * ICalCalendarParser inside a CalendarSource.
 * 
 * @author richard
 *
 */
public class CalendarSource implements ICalendarSource {

	private final IRawCalendarSource rawSource;

	private final ICalendarParser parser;

	public CalendarSource(IRawCalendarSource rawSource, ICalendarParser parser) {
		this.rawSource = rawSource;
		this.parser = parser;
	}

	private static final int PARSING_PERCENT = 30;

	public ICalendar getCalender(ILogger log, final IObserver observer) {
		final IRawCalendar rawCalendar = rawSource.getRawCalendar(log, new IRawCalendarSource.IObserver() {
			public void progress(int percent) {
				observer.progress((100 - PARSING_PERCENT) * percent / 100);
			}
		});
		observer.progress(100 - PARSING_PERCENT);
		return parser.parse(rawCalendar);
	}
}
