package calendar;

import java.util.Date;
import java.util.Vector;

import log.ILogger;
import calendar.objectmodel.ICalendar;
import calendar.objectmodel.IEventInstance;

/**
 * Combines multiple ICalendars into one
 * 
 * @author richard
 *
 */
public class CombiningCalendar implements ICalendar {

	private final ICalendar[] calendars;

	public CombiningCalendar(ICalendar[] calendars) {
		this.calendars = calendars;
	}

	public IEventInstance[] getEvents(Date date, ILogger log) {
		return getEvents(date, calendars, log);
	}

	public static IEventInstance[] getEvents(Date date, ICalendar[] calendars, ILogger log) {
		Vector events = new Vector();
		for (int i = 0; i < calendars.length; i++) {
			final IEventInstance[] eventsFromCalendar = calendars[i].getEvents(date, log);

			for (int e = 0; e < eventsFromCalendar.length; e++) {
				events.addElement(eventsFromCalendar[e]);
			}
		}
		return (IEventInstance[]) OldJavaUtils.toArray(events, new IEventInstance[events.size()]);
	}

	public Date getAsOf() {
		Date earliestDate = new Date();
		for (int i = 0; i < calendars.length; i++) {
			final Date asOfForCalendar = calendars[i].getAsOf();
			if (earliestDate.getTime() > asOfForCalendar.getTime())
				earliestDate = asOfForCalendar;
		}
		return earliestDate;
	}
}
