package calendar;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;

import javax.microedition.io.Connector;
import javax.microedition.io.file.FileConnection;

import log.ILogger;

public class FileConnectionCalendarSource implements IRawCalendarSource {

	private final String URL;

	public FileConnectionCalendarSource(String filename) {
		this.URL = filename;
	}

	public IRawCalendar getRawCalendar(ILogger log, IObserver observer) {

		try {
			log.log("Attempting to open file " + URL, ILogger.INFO);
			final FileConnection fileconnection = (FileConnection) Connector.open(URL);

			final InputStream fis = fileconnection.openInputStream();
			final long overallSize = fileconnection.fileSize();
			final Date asof = new Date(fileconnection.lastModified());
			return new InputStreamCalendarSource(fis, overallSize, asof).getRawCalendar(log, observer);
		} catch (IOException e) {
			throw new WrappingException("Error loading raw calendar", e);
		}
	}

	public void saveRawCalendar(ILogger log, IRawCalendar rawCalendar) {
		FileConnection fileconnection = null;
		OutputStream outputStream = null;
		try {
			log.log("Attempting to open " + URL, ILogger.INFO);
			fileconnection = (FileConnection) Connector.open(URL);
			if (!fileconnection.exists()) {
				fileconnection.create();
			}
			outputStream = fileconnection.openOutputStream();

			log.log("Writing " + rawCalendar.getData().length(), ILogger.INFO);

			outputStream.write(rawCalendar.getData().getBytes());
		} catch (IOException e) {
			log.log("Error saving calendar " + e.getMessage(), ILogger.ERROR);
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		} finally {
			try {
				if (outputStream != null)
					outputStream.close();
				if (fileconnection != null)
					fileconnection.close();
			} catch (IOException e) {
				log.log("Error closing output Streams", ILogger.ERROR);
				throw new RuntimeException(e.getMessage());
			}
		}
	}
}
