package calendar;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Date;

import javax.microedition.io.Connector;
import javax.microedition.io.HttpConnection;

import log.ILogger;

public class HttpConnectionCalendarSource implements IRawCalendarSource {

	private final String filename;

	public HttpConnectionCalendarSource(String filename) {
		this.filename = filename;
	}

	public IRawCalendar getRawCalendar(ILogger log, IObserver observer) {

		final String URL = "http://" + filename;
		log.log("Attempting to open http " + URL, ILogger.INFO);

		final Date startingLoad = new Date();

		HttpConnection connection = null;
		InputStream inputstream = null;
		try {
			connection = (HttpConnection) Connector.open(URL);
			// HTTP Request
			connection.setRequestMethod(HttpConnection.GET);
			connection.setRequestProperty("Content-Type", "//text plain");
			connection.setRequestProperty("Connection", "close");
			// HTTP Response
			if (connection.getResponseCode() == HttpConnection.HTTP_OK) {
				final String str;
				inputstream = connection.openInputStream();
				int length = (int) connection.getLength();
				if (length != -1) {
					byte incomingData[] = new byte[length];
					inputstream.read(incomingData);
					str = new String(incomingData);
				} else {
					ByteArrayOutputStream bytestream = new ByteArrayOutputStream();
					int ch;
					while ((ch = inputstream.read()) != -1) {
						bytestream.write(ch);
					}
					str = new String(bytestream.toByteArray());
					bytestream.close();
				}
				log.log("Successfully Read " + str.length(), ILogger.INFO);
				return new IRawCalendar() {
					public String getData() {
						return str;
					}

					public Date getAsOf() {
						return startingLoad;
					}
				};
			} else {
				throw new RuntimeException("Error loading http: " + connection.getResponseCode());
			}
		} catch (IOException error) {
			throw new RuntimeException(error.getMessage());
		} finally {
			String totalErrors = "";
			if (inputstream != null) {
				try {
					inputstream.close();
				} catch (Exception error) {
					totalErrors += error.getMessage();
				}
			}
			if (connection != null) {
				try {
					connection.close();
				} catch (Exception error) {
					totalErrors += error.getMessage();
				}
			}
			if (!"".equals(totalErrors)) {
				throw new RuntimeException(totalErrors);
			}
		}
	}
}
