package calendar;

import java.io.IOException;
import java.io.InputStream;
import java.util.Date;
import java.util.Vector;

import log.ILogger;

/**
 * This shouldn't really be an IRawCalendarSource - it confuses the issue of ownership
 * (and closing) of the InputStream. This should probably just be a static method, for
 * convenience use by the HTTP and File sources.
 * 
 * @author richard
 *
 */
public class InputStreamCalendarSource implements IRawCalendarSource {

	private final InputStream inputStream;
	private final long expectedSize;
	private final Date asof;

	public InputStreamCalendarSource(InputStream inputStream, long expectedSize, Date asof) {
		this.inputStream = inputStream;
		this.expectedSize = expectedSize;
		this.asof = asof;
	}

	public IRawCalendar getRawCalendar(ILogger log, IObserver observer) {
		final Vector buffers = new Vector();
		int totalLength = 0;

		try {
			for (;;) {
				final byte[] tempBuffer = new byte[1024];
				final int bytesRead = inputStream.read(tempBuffer, 0, tempBuffer.length);
				if (bytesRead == -1)
					break;
				final byte[] fullBuffer;
				if (bytesRead < tempBuffer.length) {
					fullBuffer = new byte[bytesRead];
					System.arraycopy(tempBuffer, 0, fullBuffer, 0, bytesRead);
				} else {
					fullBuffer = tempBuffer;
				}
				totalLength += fullBuffer.length;
				buffers.addElement(fullBuffer);
				observer.progress((int) (totalLength * 100.0 / expectedSize));
			}
			inputStream.close();
		} catch (IOException e) {
			throw new WrappingException("Error loading calendar from input stream", e);
		}

		final byte[] combined = new byte[totalLength];
		int offset = 0;
		for (int i = 0; i < buffers.size(); i++) {
			byte[] tempBuffer = (byte[]) buffers.elementAt(i);
			System.arraycopy(tempBuffer, 0, combined, offset, tempBuffer.length);
			offset += tempBuffer.length;
		}

		final String combinedString = new String(combined);

		return new IRawCalendar() {
			public String getData() {
				return combinedString;
			}

			public Date getAsOf() {
				return asof;
			}
		};
	}
}
