package calendar;

import log.ILogger;

public class LocallyCachingCalendarSource {

	private final IRawCalendarSource original;

	private final FileConnectionCalendarSource localFileCache;

	public LocallyCachingCalendarSource(String name, IRawCalendarSource original) {
		this.original = original;
		localFileCache = new FileConnectionCalendarSource(name);
	}

	public final IRawCalendarSource localCache = new IRawCalendarSource() {
		public IRawCalendar getRawCalendar(ILogger log, IObserver observer) {
			synchronized (localFileCache) {
				return localFileCache.getRawCalendar(log, observer);
			}
		}
	};

	public final IRawCalendarSource remoteVersion = new IRawCalendarSource() {
		public IRawCalendar getRawCalendar(ILogger log, IObserver observer) {
			final IRawCalendar remoteCalendar = original.getRawCalendar(log, observer);
			synchronized (localFileCache) {
				try {
					localFileCache.saveRawCalendar(log, remoteCalendar);
				} catch (Exception e) {
					throw new RuntimeException("Failed to save local cache: " + e.getMessage());
				}
			}
			return remoteCalendar;
		}
	};

	public String getName(String name) {
		return "Cached(" + name + original + ")";
	}
}
