package calendar;

import java.util.Date;

import calendar.objectmodel.IEventDescription;
import calendar.objectmodel.IEventInstance;

/**
 * Represents the repeat instances of an event
 * 
 * @author richard
 *
 */
public class RepeatEvent implements IEventInstance {

	private final IEventDescription original;

	private final Date newStart;

	public RepeatEvent(IEventDescription original, Date newStart) {
		this.original = original;
		this.newStart = newStart;
	}

	public String getTitle() {
		return original.getTitle();
	}

	public Date getStart() {
		return newStart;
	}

	public Date getEnd() {
		long length = original.getEnd().getTime() - original.getStart().getTime();
		return new Date(length + newStart.getTime());
	}

	public String getDescription() {
		return original.getDescription();
	}

	public String getLocation() {
		return original.getLocation();
	}

	public String toString() {
		return "Repeat[" + newStart + ", " + original + "]";
	}
}
