package calendar;

import java.util.Calendar;
import java.util.TimeZone;

import log.ILogger;
import log.Logger;

// Hacked up class to help deal with some strange timezone behaviour on my Nokia 6013
// Remove if possible
public class TimeZonesManager {

	private final static ILogger logger = Logger.get(TimeZonesManager.class);
	
	private static final int ONE_MINUTE = 60 * 1000;
	private static final int ONE_HOUR = 60 * ONE_MINUTE;
	private static final int ONE_DAY = 24 * ONE_HOUR;

	public static final TimeZone UTC = TimeZone.getTimeZone("UTC");
	public static final TimeZone EuropeLondon = new MyTimeZoneImpl(0 * ONE_HOUR, "Europe/London", Calendar.MARCH, -1,
			Calendar.SUNDAY, 1 * ONE_HOUR, Calendar.OCTOBER, -1, Calendar.SUNDAY, 1 * ONE_HOUR, 1 * ONE_HOUR);

	public static TimeZone get(String possibleTimezone) {
		if (UTC.getID().equals(possibleTimezone))
			return UTC;
		else if (EuropeLondon.getID().equals(possibleTimezone))
			return EuropeLondon;
		else {
			logger.log("Unknown timezone " + possibleTimezone, ILogger.ERROR);
			return TimeZone.getDefault();
		}
	}
}

/*
*
* Copyright  1990-2006 Sun Microsystems, Inc. All Rights Reserved.
* DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License version
* 2 only, as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License version 2 for more details (a copy is
* included at /legal/license.txt).
*
* You should have received a copy of the GNU General Public License
* version 2 along with this work; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA
*
* Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
* Clara, CA 95054 or visit www.sun.com if you need additional
* information or have any questions.
*/

/**
 * This class provides the time zone implementations
 * for J2ME CLDC/MIDP.  By default, the only supported
 * time zone is UTC/GMT.  Vendor-specific implementations
 * may provide additional time zones.
 *
 * @see java.util.TimeZone
 */

// Copied straight from Sun code, released until GPL 2. Would like to replace!

class MyTimeZoneImpl extends TimeZone {

	private final static ILogger logger = Logger.get(MyTimeZoneImpl.class);

	static String HOME_ID = null;

	public MyTimeZoneImpl() {
	}

	/**
	 * Constructs a TimeZone with the given base time zone offset from GMT and
	 * time zone ID. Timezone IDs can be obtained from TimeZone.getAvailableIDs.
	 * Normally you should use TimeZone.getDefault to construct a TimeZone.
	 * 
	 * @param rawOffset
	 *            The given base time zone offset to GMT.
	 * @param ID
	 *            The time zone ID which is obtained from
	 *            TimeZone.getAvailableIDs.
	 */
	private MyTimeZoneImpl(int rawOffset, String ID) {
		this.rawOffset = rawOffset;
		this.ID = ID;
		dstSavings = millisPerHour; // In case user sets rules later
	}

	/**
	 * Constructor. This constructor is identical to the 10-argument
	 * constructor, but also takes a dstSavings parameter.
	 * 
	 * @param dstSavings
	 *            The amount of time in ms saved during DST.
	 * @exception IllegalArgumentException
	 *                the month, day, dayOfWeek, or time parameters are out of
	 *                range for the start or end rule
	 */
	MyTimeZoneImpl(int rawOffset, String ID, int startMonth, int startDay, int startDayOfWeek, int startTime,
			int endMonth, int endDay, int endDayOfWeek, int endTime, int dstSavings) {
		this.ID = ID;
		this.rawOffset = rawOffset;
		this.startMonth = startMonth;
		this.startDay = startDay;
		this.startDayOfWeek = startDayOfWeek;
		this.startTime = startTime;
		this.endMonth = endMonth;
		this.endDay = endDay;
		this.endDayOfWeek = endDayOfWeek;
		this.endTime = endTime;
		this.dstSavings = dstSavings;
		decodeRules();
		if (dstSavings <= 0) {
			throw new IllegalArgumentException("Illegal DST savings");
		}
	}

	/* Constants used internally; unit is milliseconds */
	private static final int ONE_MINUTE = 60 * 1000;
	private static final int ONE_HOUR = 60 * ONE_MINUTE;
	private static final int ONE_DAY = 24 * ONE_HOUR;

	/**
	 * Gets offset, for current date, modified in case of daylight savings. This
	 * is the offset to add *to* GMT to get local time. Gets the time zone
	 * offset, for current date, modified in case of daylight savings. This is
	 * the offset to add *to* GMT to get local time. Assume that the start and
	 * end month are distinct. This method may return incorrect results for
	 * rules that start at the end of February (e.g., last Sunday in February)
	 * or the beginning of March (e.g., March 1).
	 * 
	 * @param era
	 *            The era of the given date (0 = BC, 1 = AD).
	 * @param year
	 *            The year in the given date.
	 * @param month
	 *            The month in the given date. Month is 0-based. e.g., 0 for
	 *            January.
	 * @param day
	 *            The day-in-month of the given date.
	 * @param dayOfWeek
	 *            The day-of-week of the given date.
	 * @param millis
	 *            The milliseconds in day in <em>standard</em> local time.
	 * @return The offset to add *to* GMT to get local time.
	 * @exception IllegalArgumentException
	 *                the era, month, day, dayOfWeek, or millis parameters are
	 *                out of range
	 */
	public int getOffset(int era, int year, int month, int day, int dayOfWeek, int millis) {
		if (month < Calendar.JANUARY || month > Calendar.DECEMBER) {

			throw new IllegalArgumentException("Illegal month " + month);
		}
		return getOffset(era, year, month, day, dayOfWeek, millis, staticMonthLength[month]);
	}

	/**
	 * Gets offset, for current date, modified in case of daylight savings. This
	 * is the offset to add <em>to</em> GMT to get local time. Gets the time
	 * zone offset, for current date, modified in case of daylight savings. This
	 * is the offset to add *to* GMT to get local time. Assume that the start
	 * and end month are distinct.
	 * 
	 * @param era
	 *            The era of the given date (0 = BC, 1 = AD).
	 * @param year
	 *            The year in the given date.
	 * @param month
	 *            The month in the given date. Month is 0-based. e.g., 0 for
	 *            January.
	 * @param day
	 *            The day-in-month of the given date.
	 * @param dayOfWeek
	 *            The day-of-week of the given date.
	 * @param millis
	 *            The milliseconds in day in <em>standard</em> local time.
	 * @param monthLength
	 *            The length of the given month in days.
	 * @return The offset to add *to* GMT to get local time.
	 * @exception IllegalArgumentException
	 *                the era, month, day, dayOfWeek, millis, or monthLength
	 *                parameters are out of range
	 */
	int getOffset(int era, int year, int month, int day, int dayOfWeek, int millis, int monthLength) {
		logger.log("getOffset " + era + " " + year + " " + month + " " + day + " " + dayOfWeek + " " + millis + " "
				+ monthLength, ILogger.DEBUG);
		if (true) {
			// Use this parameter checking code for normal operation. Only one
			// of these two blocks should actually get compiled into the class
			// file.
			if ((era != 0 && era != 1) || month < Calendar.JANUARY || month > Calendar.DECEMBER || day < 1
					|| day > monthLength || dayOfWeek < Calendar.SUNDAY || dayOfWeek > Calendar.SATURDAY || millis < 0
					|| millis >= millisPerDay || monthLength < 28 || monthLength > 31) {

				throw new IllegalArgumentException();
			}
		} else {
			// This parameter checking code is better for debugging, but
			// overkill for normal operation. Only one of these two blocks
			// should actually get compiled into the class file.
			if (era != 0 && era != 1) {
				throw new IllegalArgumentException("Illegal era " + era);
			}
			if (month < Calendar.JANUARY || month > Calendar.DECEMBER) {
				throw new IllegalArgumentException("Illegal month " + month);
			}
			if (day < 1 || day > monthLength) {
				throw new IllegalArgumentException("Illegal day " + day);
			}
			if (dayOfWeek < Calendar.SUNDAY || dayOfWeek > Calendar.SATURDAY) {
				throw new IllegalArgumentException("Illegal day of week " + dayOfWeek);
			}
			if (millis < 0 || millis >= millisPerDay) {
				throw new IllegalArgumentException("Illegal millis " + millis);
			}
			if (monthLength < 28 || monthLength > 31) {
				throw new IllegalArgumentException("Illegal month length " + monthLength);
			}
		}

		int result = rawOffset;

		// Bail out if we are before the onset of daylight savings time
		if (!useDaylight || year < startYear || era != 1)
			return result;

		// Check for southern hemisphere. We assume that the start and end
		// month are different.
		boolean southern = (startMonth > endMonth);

		// Compare the date to the starting and ending rules.+1 = date>rule, -1
		// = date<rule, 0 = date==rule.
		int startCompare = compareToRule(month, monthLength, day, dayOfWeek, millis, startMode, startMonth,
				startDayOfWeek, startDay, startTime);
		int endCompare = 0;

		// We don't always have to compute endCompare. For many instances,
		// startCompare is enough to determine if we are in DST or not. In the
		// northern hemisphere, if we are before the start rule, we can't have
		// DST. In the southern hemisphere, if we are after the start rule, we
		// must have DST. This is reflected in the way the next if statement
		// (not the one immediately following) short circuits.
		if (southern != (startCompare >= 0)) {
			// For the ending rule comparison, we add the dstSavings to the
			// millis
			// passed in to convert them from standard to wall time. We then
			// must
			// normalize the millis to the range 0..millisPerDay-1.
			millis += dstSavings; // Assume dstSavings > 0
			while (millis >= millisPerDay) {
				millis -= millisPerDay;
				++day;
				dayOfWeek = 1 + (dayOfWeek % 7); // Assume dayOfWeek is
				// one-based
				if (day > monthLength) {
					day = 1;
					// When incrementing the month, it is desirable to overflow
					// from DECEMBER to DECEMBER+1, since we use the result to
					// compare against a real month. Wraparound of the value
					// leads to bug 4173604.
					++month;
				}
			}
			endCompare = compareToRule(month, monthLength, day, dayOfWeek, millis, endMode, endMonth, endDayOfWeek,
					endDay, endTime);
		}

		// Check for both the northern and southern hemisphere cases. We
		// assume that in the northern hemisphere, the start rule is before the
		// end rule within the calendar year, and vice versa for the southern
		// hemisphere.
		if ((!southern && (startCompare >= 0 && endCompare < 0)) || (southern && (startCompare >= 0 || endCompare < 0))) {

			result += dstSavings;
		}

		logger.log("getoffset " + result, ILogger.DEBUG);

		return result;
	}

	/**
	 * Compare a given date in the year to a rule. Return 1, 0, or -1, depending
	 * on whether the date is after, equal to, or before the rule date. The
	 * millis are compared directly against the ruleMillis, so any
	 * standard-daylight adjustments must be handled by the caller.
	 * 
	 * @return 1 if the date is after the rule date, -1 if the date is before
	 *         the rule date, or 0 if the date is equal to the rule date.
	 */
	private static int compareToRule(int month, int monthLen, int dayOfMonth, int dayOfWeek, int millis, int ruleMode,
			int ruleMonth, int ruleDayOfWeek, int ruleDay, int ruleMillis) {
		if (month < ruleMonth)
			return -1;
		else if (month > ruleMonth)
			return 1;

		int ruleDayOfMonth = 0;
		switch (ruleMode) {
		case DOM_MODE:
			ruleDayOfMonth = ruleDay;
			break;
		case DOW_IN_MONTH_MODE:
			// In this case ruleDay is the day-of-week-in-month
			if (ruleDay > 0) {
				ruleDayOfMonth = 1 + (ruleDay - 1) * 7 + (7 + ruleDayOfWeek - (dayOfWeek - dayOfMonth + 1)) % 7;
			} else {
				// Assume ruleDay < 0 here
				ruleDayOfMonth = monthLen + (ruleDay + 1) * 7
						- (7 + (dayOfWeek + monthLen - dayOfMonth) - ruleDayOfWeek) % 7;
			}
			break;
		case DOW_GE_DOM_MODE:
			ruleDayOfMonth = ruleDay + (49 + ruleDayOfWeek - ruleDay - dayOfWeek + dayOfMonth) % 7;
			break;
		case DOW_LE_DOM_MODE:
			ruleDayOfMonth = ruleDay - (49 - ruleDayOfWeek + ruleDay + dayOfWeek - dayOfMonth) % 7;
			// Note at this point ruleDayOfMonth may be <1, although it will
			// be >=1 for well-formed rules.
			break;
		}

		if (dayOfMonth < ruleDayOfMonth)
			return -1;
		else if (dayOfMonth > ruleDayOfMonth)
			return 1;

		if (millis < ruleMillis)
			return -1;
		else if (millis > ruleMillis)
			return 1;
		else
			return 0;
	}

	/**
	 * Gets the GMT offset for this time zone.
	 */
	public int getRawOffset() {
		// The given date will be taken into account while
		// we have the historical time zone data in place.
		return rawOffset;
	}

	/**
	 * Queries if this time zone uses Daylight Savings Time.
	 */
	public boolean useDaylightTime() {
		return useDaylight;
	}

	/**
	 * Gets the ID of this time zone.
	 * 
	 * @return the ID of this time zone.
	 */
	public String getID() {
		return ID;
	}

	// =======================privates===============================

	/**
	 * The string identifier of this <code>TimeZone</code>. This is a
	 * programmatic identifier used internally to look up <code>TimeZone</code>
	 * objects from the system table and also to map them to their localized
	 * display names. <code>ID</code> values are unique in the system table but
	 * may not be for dynamically created zones.
	 * 
	 * @serial
	 */
	private String ID;

	static String[] ids = null;

	/**
	 * The month in which daylight savings time starts. This value must be
	 * between <code>Calendar.JANUARY</code> and <code>Calendar.DECEMBER</code>
	 * inclusive. This value must not equal <code>endMonth</code>.
	 * <p>
	 * If <code>useDaylight</code> is false, this value is ignored.
	 * 
	 * @serial
	 */
	private int startMonth;

	/**
	 * This field has two possible interpretations:
	 * <dl>
	 * <dt><code>startMode == DOW_IN_MONTH</code></dt>
	 * <dd>
	 * <code>startDay</code> indicates the day of the month of
	 * <code>startMonth</code> on which daylight savings time starts, from 1 to
	 * 28, 30, or 31, depending on the <code>startMonth</code>.</dd>
	 * <dt><code>startMode != DOW_IN_MONTH</code></dt>
	 * <dd>
	 * <code>startDay</code> indicates which <code>startDayOfWeek</code> in th
	 * month <code>startMonth</code> daylight savings time starts on. For
	 * example, a value of +1 and a <code>startDayOfWeek</code> of
	 * <code>Calendar.SUNDAY</code> indicates the first Sunday of
	 * <code>startMonth</code>. Likewise, +2 would indicate the second Sunday,
	 * and -1 the last Sunday. A value of 0 is illegal.</dd>
	 * </ul>
	 * <p>
	 * If <code>useDaylight</code> is false, this value is ignored.
	 * 
	 * @serial
	 */
	private int startDay;

	/**
	 * The day of the week on which daylight savings time starts. This value
	 * must be between <code>Calendar.SUNDAY</code> and
	 * <code>Calendar.SATURDAY</code> inclusive.
	 * <p>
	 * If <code>useDaylight</code> is false or
	 * <code>startMode == DAY_OF_MONTH</code>, this value is ignored.
	 * 
	 * @serial
	 */
	private int startDayOfWeek;

	/**
	 * The time in milliseconds after midnight at which daylight savings time
	 * starts. This value is expressed as <em>wall time</em>, which means it is
	 * compared to <em>standard</em> time for the daylight savings start.
	 * <p>
	 * If <code>useDaylight</code> is false, this value is ignored.
	 * 
	 * @serial
	 */
	private int startTime;

	/**
	 * The month in which daylight savings time ends. This value must be between
	 * <code>Calendar.JANUARY</code> and <code>Calendar.UNDECIMBER</code>. This
	 * value must not equal <code>startMonth</code>.
	 * <p>
	 * If <code>useDaylight</code> is false, this value is ignored.
	 * 
	 * @serial
	 */
	private int endMonth;

	/**
	 * This field has two possible interpretations:
	 * <dl>
	 * <dt><code>endMode == DOW_IN_MONTH</code></dt>
	 * <dd>
	 * <code>endDay</code> indicates the day of the month of
	 * <code>endMonth</code> on which daylight savings time ends, from 1 to 28,
	 * 30, or 31, depending on the <code>endMonth</code>.</dd>
	 * <dt><code>endMode != DOW_IN_MONTH</code></dt>
	 * <dd>
	 * <code>endDay</code> indicates which <code>endDayOfWeek</code> in th month
	 * <code>endMonth</code> daylight savings time ends on. For example, a value
	 * of +1 and a <code>endDayOfWeek</code> of <code>Calendar.SUNDAY</code>
	 * indicates the first Sunday of <code>endMonth</code>. Likewise, +2 would
	 * indicate the second Sunday, and -1 the last Sunday. A value of 0 is
	 * illegal.</dd>
	 * </ul>
	 * <p>
	 * If <code>useDaylight</code> is false, this value is ignored.
	 * 
	 * @serial
	 */
	private int endDay;

	/**
	 * The day of the week on which daylight savings time ends. This value must
	 * be between <code>Calendar.SUNDAY</code> and
	 * <code>Calendar.SATURDAY</code> inclusive.
	 * <p>
	 * If <code>useDaylight</code> is false or
	 * <code>endMode == DAY_OF_MONTH</code>, this value is ignored.
	 * 
	 * @serial
	 */
	private int endDayOfWeek;

	/**
	 * The time in milliseconds after midnight at which daylight savings time
	 * ends. This value is expressed as <em>wall time</em>, which means it is
	 * compared to <em>daylight</em> time for the daylight savings end.
	 * <p>
	 * If <code>useDaylight</code> is false, this value is ignored.
	 * 
	 * @serial
	 */
	private int endTime;

	/**
	 * The year in which daylight savings time is first observed. This is an AD
	 * value. If this value is less than 1 then daylight savings is observed for
	 * all AD years.
	 * <p>
	 * If <code>useDaylight</code> is false, this value is ignored.
	 * 
	 * @serial
	 */
	private int startYear;

	/**
	 * The offset in milliseconds between this zone and GMT. Negative offsets
	 * are to the west of Greenwich. To obtain local <em>standard</em> time, add
	 * the offset to GMT time. To obtain local wall time it may also be
	 * necessary to add <code>dstSavings</code>.
	 * 
	 * @serial
	 */
	private int rawOffset;

	/**
	 * A boolean value which is true if and only if this zone uses daylight
	 * savings time. If this value is false, several other fields are ignored.
	 * 
	 * @serial
	 */
	private boolean useDaylight = false; // indicate if this time zone uses DST

	private static final int millisPerHour = 60 * 60 * 1000;
	private static final int millisPerDay = 24 * millisPerHour;

	/**
	 * This field was serialized in JDK 1.1, so we have to keep it that way to
	 * maintain serialization compatibility. However, there's no need to
	 * recreate the array each time we create a new time zone.
	 * 
	 * @serial An array of bytes containing the values {31, 28, 31, 30, 31, 30,
	 *         31, 31, 30, 31, 30, 31}. This is ignored as of JDK 1.2, however,
	 *         it must be streamed out for compatibility with JDK 1.1.
	 */
	private final byte monthLength[] = staticMonthLength;
	private final static byte staticMonthLength[] = { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 }; // **NS**

	/**
	 * Variables specifying the mode of the start rule. Takes the following
	 * values:
	 * <dl>
	 * <dt><code>DOM_MODE</code></dt>
	 * <dd>
	 * Exact day of week; e.g., March 1.</dd>
	 * <dt><code>DOW_IN_MONTH_MODE</code></dt>
	 * <dd>
	 * Day of week in month; e.g., last Sunday in March.</dd>
	 * <dt><code>DOW_GE_DOM_MODE</code></dt>
	 * <dd>
	 * Day of week after day of month; e.g., Sunday on or after March 15.</dd>
	 * <dt><code>DOW_LE_DOM_MODE</code></dt>
	 * <dd>
	 * Day of week before day of month; e.g., Sunday on or before March 15.</dd>
	 * </dl>
	 * The setting of this field affects the interpretation of the
	 * <code>startDay</code> field.
	 * <p>
	 * If <code>useDaylight</code> is false, this value is ignored.
	 * 
	 * @serial
	 * @since JDK1.1.4
	 */
	private int startMode;

	/**
	 * Variables specifying the mode of the end rule. Takes the following
	 * values:
	 * <dl>
	 * <dt><code>DOM_MODE</code></dt>
	 * <dd>
	 * Exact day of week; e.g., March 1.</dd>
	 * <dt><code>DOW_IN_MONTH_MODE</code></dt>
	 * <dd>
	 * Day of week in month; e.g., last Sunday in March.</dd>
	 * <dt><code>DOW_GE_DOM_MODE</code></dt>
	 * <dd>
	 * Day of week after day of month; e.g., Sunday on or after March 15.</dd>
	 * <dt><code>DOW_LE_DOM_MODE</code></dt>
	 * <dd>
	 * Day of week before day of month; e.g., Sunday on or before March 15.</dd>
	 * </dl>
	 * The setting of this field affects the interpretation of the
	 * <code>endDay</code> field.
	 * <p>
	 * If <code>useDaylight</code> is false, this value is ignored.
	 * 
	 * @serial
	 * @since JDK1.1.4
	 */
	private int endMode;

	/**
	 * A positive value indicating the amount of time saved during DST in
	 * milliseconds. Typically one hour (3600000); sometimes 30 minutes
	 * (1800000).
	 * <p>
	 * If <code>useDaylight</code> is false, this value is ignored.
	 * 
	 * @serial
	 * @since JDK1.1.4
	 */
	private int dstSavings;

	/**
	 * Constants specifying values of startMode and endMode.
	 */
	private static final int DOM_MODE = 1; // Exact day of month, "Mar 1"
	private static final int DOW_IN_MONTH_MODE = 2; // Day of week in month,
	// "lastSun"
	private static final int DOW_GE_DOM_MODE = 3; // Day of week after day of
	// month, "Sun>=15"
	private static final int DOW_LE_DOM_MODE = 4; // Day of week before day of

	// month, "Sun<=21"

	// ----------------------------------------------------------------------
	// Rule representation
	//
	// We represent the following flavors of rules:
	// 5 the fifth of the month
	// lastSun the last Sunday in the month
	// lastMon the last Monday in the month
	// Sun>=8 first Sunday on or after the eighth
	// Sun<=25 last Sunday on or before the 25th
	// This is further complicated by the fact that we need to remain
	// backward compatible with the 1.1 FCS. Finally, we need to minimize
	// API changes. In order to satisfy these requirements, we support
	// three representation systems, and we translate between them.
	//
	// INTERNAL REPRESENTATION
	// This is the format TimeZone objects take after construction or
	// streaming in is complete. Rules are represented directly, using an
	// unencoded format. We will discuss the start rule only below; the end
	// rule is analogous.
	// startMode Takes on enumerated values DAY_OF_MONTH,
	// DOW_IN_MONTH, DOW_AFTER_DOM, or DOW_BEFORE_DOM.
	// startDay The day of the month, or for DOW_IN_MONTH mode, a
	// value indicating which DOW, such as +1 for first,
	// +2 for second, -1 for last, etc.
	// startDayOfWeek The day of the week. Ignored for DAY_OF_MONTH.
	//
	// ENCODED REPRESENTATION
	// This is the format accepted by the constructor and by setStartRule()
	// and setEndRule(). It uses various combinations of positive, negative,
	// and zero values to encode the different rules. This representation
	// allows us to specify all the different rule flavors without altering
	// the API.
	// MODE startMonth startDay startDayOfWeek
	// DOW_IN_MONTH_MODE >=0 !=0 >0
	// DOM_MODE >=0 >0 ==0
	// DOW_GE_DOM_MODE >=0 >0 <0
	// DOW_LE_DOM_MODE >=0 <0 <0
	// (no DST) don't care ==0 don't care
	//
	// STREAMED REPRESENTATION
	// We must retain binary compatibility with the 1.1 FCS. The 1.1 code only
	// handles DOW_IN_MONTH_MODE and non-DST mode, the latter indicated by the
	// flag useDaylight. When we stream an object out, we translate into an
	// approximate DOW_IN_MONTH_MODE representation so the object can be parsed
	// and used by 1.1 code. Following that, we write out the full
	// representation separately so that contemporary code can recognize and
	// parse it. The full representation is written in a "packed" format,
	// consisting of a version number, a length, and an array of bytes. Future
	// versions of this class may specify different versions. If they wish to
	// include additional data, they should do so by storing them after the
	// packed representation below.
	// ----------------------------------------------------------------------

	/**
	 * Given a set of encoded rules in startDay and startDayOfMonth, decode them
	 * and set the startMode appropriately. Do the same for endDay and
	 * endDayOfMonth. Upon entry, the day of week variables may be zero or
	 * negative, in order to indicate special modes. The day of month variables
	 * may also be negative. Upon exit, the mode variables will be set, and the
	 * day of week and day of month variables will be positive. This method also
	 * recognizes a startDay or endDay of zero as indicating no DST.
	 */
	private void decodeRules() {
		decodeStartRule();
		decodeEndRule();
	}

	/**
	 * Decode the start rule and validate the parameters. The parameters are
	 * expected to be in encoded form, which represents the various rule modes
	 * by negating or zeroing certain values. Representation formats are:
	 * <p>
	 * 
	 * <pre>
	 *            DOW_IN_MONTH  DOM    DOW>=DOM  DOW<=DOM  no DST
	 *            ------------  -----  --------  --------  ----------
	 * month       0..11        same    same      same     don't care
	 * day        -5..5         1..31   1..31    -1..-31   0
	 * dayOfWeek   1..7         0      -1..-7    -1..-7    don't care
	 * time        0..ONEDAY    same    same      same     don't care
	 * </pre>
	 * 
	 * The range for month does not include UNDECIMBER since this class is
	 * really specific to Calendar, which does not use that month. The range for
	 * time includes ONEDAY (vs. ending at ONEDAY-1) because the end rule is an
	 * exclusive limit point. That is, the range of times that are in DST
	 * include those >= the start and < the end. For this reason, it should be
	 * possible to specify an end of ONEDAY in order to include the entire day.
	 * Although this is equivalent to time 0 of the following day, it's not
	 * always possible to specify that, for example, on December 31. While
	 * arguably the start range should still be 0..ONEDAY-1, we keep the start
	 * and end ranges the same for consistency.
	 */
	private void decodeStartRule() {
		useDaylight = (startDay != 0) && (endDay != 0);
		if (startDay != 0) {
			if (startMonth < Calendar.JANUARY || startMonth > Calendar.DECEMBER) {
				throw new IllegalArgumentException("Illegal start month " + startMonth);
			}
			if (startTime < 0 || startTime > millisPerDay) {
				throw new IllegalArgumentException("Illegal start time " + startTime);
			}
			if (startDayOfWeek == 0) {
				startMode = DOM_MODE;
			} else {
				if (startDayOfWeek > 0) {
					startMode = DOW_IN_MONTH_MODE;
				} else {
					startDayOfWeek = -startDayOfWeek;
					if (startDay > 0) {
						startMode = DOW_GE_DOM_MODE;
					} else {
						startDay = -startDay;
						startMode = DOW_LE_DOM_MODE;
					}
				}
				if (startDayOfWeek > Calendar.SATURDAY) {
					throw new IllegalArgumentException("Illegal start day of week " + startDayOfWeek);
				}
			}
			if (startMode == DOW_IN_MONTH_MODE) {
				if (startDay < -5 || startDay > 5) {
					throw new IllegalArgumentException("Illegal start day of week in month " + startDay);
				}
			} else if (startDay > staticMonthLength[startMonth]) {
				throw new IllegalArgumentException("Illegal start day " + startDay);
			}
		}
	}

	/**
	 * Decode the end rule and validate the parameters. This method is exactly
	 * analogous to decodeStartRule().
	 * 
	 * @see decodeStartRule
	 */
	private void decodeEndRule() {
		useDaylight = (startDay != 0) && (endDay != 0);
		if (endDay != 0) {
			if (endMonth < Calendar.JANUARY || endMonth > Calendar.DECEMBER) {
				throw new IllegalArgumentException("Illegal end month " + endMonth);
			}
			if (endTime < 0 || endTime > millisPerDay) {
				throw new IllegalArgumentException("Illegal end time " + endTime);
			}
			if (endDayOfWeek == 0) {
				endMode = DOM_MODE;
			} else {
				if (endDayOfWeek > 0) {
					endMode = DOW_IN_MONTH_MODE;
				} else {
					endDayOfWeek = -endDayOfWeek;
					if (endDay > 0) {
						endMode = DOW_GE_DOM_MODE;
					} else {
						endDay = -endDay;
						endMode = DOW_LE_DOM_MODE;
					}
				}
				if (endDayOfWeek > Calendar.SATURDAY) {
					throw new IllegalArgumentException("Illegal end day of week " + endDayOfWeek);
				}
			}
			if (endMode == DOW_IN_MONTH_MODE) {
				if (endDay < -5 || endDay > 5) {
					throw new IllegalArgumentException("Illegal end day of week in month " + endDay);
				}
			} else if (endDay > staticMonthLength[endMonth]) {
				throw new IllegalArgumentException("Illegal end day " + endDay);
			}
		}
	}
}
