package calendarsourceentries;

import log.ILogger;
import calendar.CacheFileManager;
import calendar.CalendarSource;
import calendar.HttpConnectionCalendarSource;
import calendar.ICalCalendarParser;
import calendar.ICalendarSource;
import calendar.LocallyCachingCalendarSource;

public class CalendarHttpSourceEntry extends CalendarSourceEntry {

	public static final String MARKER = "HTTP";

	private final String hostname;

	private final String filename;

	public CalendarHttpSourceEntry(String name, String hostname,
			String filename) {
		super(name);
		this.hostname = hostname;
		this.filename = filename;
	}

	public ICalendarSource[] createSources(ILogger log) {
		// FIXME: This doesn't deal with calendars changing name!
		final String localCacheName = CacheFileManager.getInstance(log)
				.createCacheNode("Remote" + getName());

		final HttpConnectionCalendarSource httpSource = new HttpConnectionCalendarSource(
				hostname + "/" + filename);
		final LocallyCachingCalendarSource locallyCaching = new LocallyCachingCalendarSource(
				localCacheName, httpSource);
		return new ICalendarSource[] {
				new CalendarSource(locallyCaching.localCache,
						ICalCalendarParser.INSTANCE),
				new CalendarSource(locallyCaching.remoteVersion,
						ICalCalendarParser.INSTANCE) };
	}

	public static CalendarHttpSourceEntry parse(String name, String[] data) {
		return new CalendarHttpSourceEntry(name, data[0], data[1]);
	}

	protected String getMarker() {
		return MARKER;
	}

	protected String[] internalSerialize() {
		return new String[] { hostname, filename };
	}

	public String getHostname() {
		return hostname;
	}

	public String getFilename() {
		return filename;
	}
}
