package phonecalendar;

import java.util.Vector;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.List;

import calendar.OldJavaUtils;
import calendarsourceentries.CalendarSourceEntry;
import calendarsourceentries.CalendarSourceHolder;

public class CalendarList extends List {

	// CalendarSourceHolder
	private Vector currentlyDisplayedCalendars;

	private FormClosedListener closedListener;

	public interface CalendarsChangedListener {
		void updatedOrCreated(CalendarSourceHolder calendar);
		void deleted(CalendarSourceHolder calendar);
	}

	public CalendarList(
			final CalendarsChangedListener calendarsChangedListener,
			final Display display) {
		super("Calendars", List.IMPLICIT);

		final Command EDIT_COMMAND = new Command("Edit", Command.OK, 0);
		final Command NEW_COMMAND = new Command("New", Command.OK, 0);
		final Command DELETE_COMMAND = new Command("Delete", Command.OK, 0);
		final Command BACK_COMMAND = new Command("Back", Command.BACK, 0);

		addCommand(EDIT_COMMAND);
		addCommand(NEW_COMMAND);
		addCommand(DELETE_COMMAND);
		addCommand(BACK_COMMAND);

		setCommandListener(new CommandListener() {
			public void commandAction(Command command, Displayable arg1) {
				final CalendarSourceHolder calendarToEdit;
				if (command == EDIT_COMMAND || command == DELETE_COMMAND) {
					calendarToEdit = getSelectedCalendar();
				} else if (command == NEW_COMMAND) {
					calendarToEdit = new CalendarSourceHolder();
					currentlyDisplayedCalendars.addElement(calendarToEdit);
				} else {
					calendarToEdit = null;
				}

				if (command == BACK_COMMAND) {
					closedListener.closed(FormClosedListener.RESULT_OK);
				}

				if (calendarToEdit != null) {
					if (command == EDIT_COMMAND || command == NEW_COMMAND) {
						final EditCalendarForm editCalendarForm = new EditCalendarForm(
								calendarToEdit);

						editCalendarForm
								.setFormClosedListener(new FormClosedListener() {
									public void closed(int result) {
										switch (result) {
										case FormClosedListener.RESULT_OK:
											calendarsChangedListener.updatedOrCreated(calendarToEdit);
											sortCalendarList();
											refreshCalendarList();
											break;
										case FormClosedListener.RESULT_CANCEL:
											break;
										}
										display.setCurrent(CalendarList.this);
									}
								});
						display.setCurrent(editCalendarForm);
					} else if (command == DELETE_COMMAND) {
						currentlyDisplayedCalendars.removeElement(calendarToEdit);
						calendarsChangedListener.deleted(calendarToEdit);
					}
				}
			}
		});
	}

	public void setFormClosedListener(FormClosedListener closedListener) {
		this.closedListener = closedListener;
	}

	private CalendarSourceHolder getSelectedCalendar() {
		if (currentlyDisplayedCalendars != null) {
			final int selectedEventIndex = getSelectedIndex();
			return (CalendarSourceHolder)currentlyDisplayedCalendars.elementAt(selectedEventIndex);
		} else
			return null;
	}

	public void populateCalendarList(Vector calendarSources) {
		currentlyDisplayedCalendars = calendarSources;

		sortCalendarList();
		refreshCalendarList();
	}
	
	private void sortCalendarList() {
		OldJavaUtils.sort(currentlyDisplayedCalendars,
				new OldJavaUtils.Comparator() {
					public int compare(Object left, Object right) {
						return ((CalendarSourceEntry) left)
								.getName()
								.compareTo(
										((CalendarSourceEntry) right).getName());
					}
				});
	}
	
	private void refreshCalendarList() {
		deleteAll();
		for (int i = 0; i < currentlyDisplayedCalendars.size(); i++) {
			final CalendarSourceHolder holder = (CalendarSourceHolder)currentlyDisplayedCalendars.elementAt(i);
			append(holder.getEntry().getName(), null);
		}	
	}
}
