package phonecalendar;

import java.util.Vector;

import calendar.CombiningCalendar;
import calendar.ICalendarSource;
import calendar.OldJavaUtils;
import calendar.objectmodel.ICalendar;
import calendarsourceentries.CalendarSourceEntry;
import calendarsourceentries.CalendarSourceEntryStore;

public class CalendarsHolder {

	private final Vector loadingCalendars = new Vector();

	public interface ILoadingCalendar {
		CalendarSourceEntry getCalendarSourceEntry();
	}

	private class LoadingCalendar implements ILoadingCalendar, Runnable {
		private boolean cancel = false;

		private final CalendarSourceEntry calendar;

		public LoadingCalendar(final CalendarSourceEntry calendar) {
			this.calendar = calendar;
		}

		public CalendarSourceEntry getCalendarSourceEntry() {
			return calendar;
		}

		public void run() {
			synchronized (CalendarsHolder.this) {
				listener.start(this);
			}
			final ICalendarSource source = calendar.createSource();
			final ICalendar loadedCalendar = source.getCalender();
			synchronized (CalendarsHolder.this) {
				listener.finish(this, loadedCalendar);
			}
		}
	}

	public interface IListener {
		void start(ILoadingCalendar loading);

		void progress(ILoadingCalendar loading, int percent);

		void finish(ILoadingCalendar loading, ICalendar calendar);
	}

	// of CalendarSourceEntry
	private Vector calendars;

	private IListener listener;

	public CalendarsHolder() {
		// Initialize from store
		calendars = CalendarSourceEntryStore.loadCalendarEntries();
	}

	public void saveCalendars() {
		CalendarSourceEntryStore.saveCalendars(calendars);
	}

	public synchronized void setListener(IListener listener) {
		for (int i = 0; i < loadingCalendars.size(); i++) {
			listener.start((ILoadingCalendar) loadingCalendars.elementAt(i));
		}
		this.listener = listener;
	}

	public void replace(CalendarSourceEntry oldCalendar, CalendarSourceEntry newCalendar) {
		final int oldIndex = calendars.indexOf(oldCalendar);
		if (oldIndex != -1) {
			calendars.setElementAt(newCalendar, oldIndex);
		} else {
			calendars.addElement(newCalendar);
		}
		startLoadingCalendar(newCalendar);
	}

	private void startLoadingCalendar(CalendarSourceEntry calendar) {
		final LoadingCalendar loader = new LoadingCalendar(calendar);
		loadingCalendars.addElement(loader);
		new Thread(loader).start();
	}

	public CalendarSourceEntry[] getCalendarSources() {
		return (CalendarSourceEntry[]) OldJavaUtils.toArray(calendars, new CalendarSourceEntry[calendars.size()]);
	}
}
