package phonecalendar;

import javax.microedition.lcdui.ChoiceGroup;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.Item;
import javax.microedition.lcdui.ItemStateListener;
import javax.microedition.lcdui.TextField;

import calendarsourceentries.CalendarFileSourceEntry;
import calendarsourceentries.CalendarHttpSourceEntry;
import calendarsourceentries.CalendarSourceEntry;
import calendarsourceentries.CalendarSourceHolder;

public class EditCalendarForm extends Form {

	private final TextField nameField = new TextField("Name:", "", 30, TextField.ANY);

	private final TextField hostname = new TextField("Hostname:", "", 30, TextField.ANY);

	private final TextField hostfilename = new TextField("Filename:", "", 256, TextField.ANY);

	private final TextField localfilename = new TextField("Filename:", "", 256, TextField.ANY);

	private final TextField localRoot = new TextField("Root:", "", 256, TextField.ANY);

	private final ChoiceGroup calendarSourceTypeSelector = new ChoiceGroup("Calendar type", ChoiceGroup.EXCLUSIVE);

	private FormClosedListener closedListener;

	private final CalendarSourceHolder calendarSource;

	public EditCalendarForm(CalendarSourceHolder calendarSource) {
		super("Edit Calendar");

		this.calendarSource = calendarSource;

		final Command OK_COMMAND = new Command("OK", Command.OK, 0);

		final Command CANCEL_COMMAND = new Command("Cancel", Command.CANCEL, 0);

		populateFormForCalendarSourceType();
		populateItemsForCalendarSourceType();
		populateCalendarSourceTypes();
		setCalendarSourceTypeIndex();

		addCommand(OK_COMMAND);
		addCommand(CANCEL_COMMAND);
		setItemStateListener(new ItemStateListener() {
			public void itemStateChanged(Item item) {
				if (item == calendarSourceTypeSelector) {
					setCalendarSourceFromItems();
					populateFormForCalendarSourceType();
				}
			}
		});

		setCommandListener(new CommandListener() {
			public void commandAction(Command command, Displayable arg1) {
				if (command == OK_COMMAND) {
					setCalendarSourceFromItems();
					closedListener.closed(FormClosedListener.RESULT_OK);
				} else if (command == CANCEL_COMMAND) {
					closedListener.closed(FormClosedListener.RESULT_CANCEL);
				}
			}
		});
	}

	public void setFormClosedListener(FormClosedListener closedListener) {
		this.closedListener = closedListener;
	}

	private void populateFormForCalendarSourceType() {
		deleteAll();
		append(nameField);
		append(calendarSourceTypeSelector);

		final CalendarSourceEntry calendarSourceEntry = calendarSource.getEntry();

		if (calendarSourceEntry instanceof CalendarFileSourceEntry) {
			append(localRoot);
			append(localfilename);
		} else if (calendarSourceEntry instanceof CalendarHttpSourceEntry) {
			append(hostname);
			append(hostfilename);
		}
	}

	private void populateItemsForCalendarSourceType() {

		final CalendarSourceEntry calendarSourceEntry = calendarSource.getEntry();

		nameField.setString(calendarSourceEntry.getName());
		if (calendarSourceEntry instanceof CalendarFileSourceEntry) {
			final CalendarFileSourceEntry fileSourceEntry = (CalendarFileSourceEntry) calendarSourceEntry;
			localfilename.setString(fileSourceEntry.getFilename());
			localRoot.setString(fileSourceEntry.getRoot());
		} else if (calendarSourceEntry instanceof CalendarHttpSourceEntry) {
			final CalendarHttpSourceEntry httpSourceEntry = (CalendarHttpSourceEntry) calendarSourceEntry;
			hostname.setString(httpSourceEntry.getHostname());
			hostfilename.setString(httpSourceEntry.getFilename());
		}
	}

	// These functions must be kept in sync
	private void populateCalendarSourceTypes() {
		calendarSourceTypeSelector.append("Http", null);
		calendarSourceTypeSelector.append("File", null);
	}

	private void setCalendarSourceFromItems() {
		int sourceIndex = calendarSourceTypeSelector.getSelectedIndex();
		switch (sourceIndex) {
		case 0: {
			calendarSource.setEntry(new CalendarHttpSourceEntry(nameField.getString(), hostname.getString(),
					hostfilename.getString()));
			break;
		}
		case 1: {
			calendarSource.setEntry(new CalendarFileSourceEntry(nameField.getString(), localRoot.getString(),
					localfilename.getString()));
			break;
		}
		default:
			throw new RuntimeException("Unknown calendar type " + sourceIndex);
		}
	}

	private void setCalendarSourceTypeIndex() {

		final CalendarSourceEntry calendarSourceEntry = calendarSource.getEntry();

		final int selectedIndex;
		if (calendarSourceEntry instanceof CalendarHttpSourceEntry)
			selectedIndex = 0;
		else if (calendarSourceEntry instanceof CalendarFileSourceEntry)
			selectedIndex = 1;
		else
			throw new RuntimeException("Unknown calendar type " + calendarSourceEntry);

		calendarSourceTypeSelector.setSelectedIndex(selectedIndex, true);
	}
}
