package phonecalendar;

import java.util.Date;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.List;

import calendar.OldJavaUtils;
import calendar.objectmodel.IEventInstance;

public class EventList extends List {
	private IEventInstance[] currentlyDisplayedEvents;

	private FormClosedListener closedListener;

	public EventList(final Display display) {
		super("Events", List.IMPLICIT);
		setFitPolicy(List.TEXT_WRAP_ON);

		final Command BACK_COMMAND = new Command("Back", Command.BACK, 0);
		addCommand(BACK_COMMAND);

		final EventDescriptionForm eventDescriptionForm = new EventDescriptionForm();

		eventDescriptionForm.setFormClosedListener(new FormClosedListener() {
			public void closed(int result) {
				display.setCurrent(EventList.this);
			}
		});

		setCommandListener(new CommandListener() {
			public void commandAction(Command command, Displayable arg1) {
				if (command == BACK_COMMAND) {
					closedListener.closed(FormClosedListener.RESULT_OK);
				} else if (command == List.SELECT_COMMAND) {
					System.out.println("Selected event");
					final IEventInstance selectedEvent = getSelectedEvent();
					if (selectedEvent != null) {
						System.out.println("Selected " + selectedEvent);
						eventDescriptionForm.setEvent(selectedEvent);
						System.out.println("Setting event desc as current form");
						display.setCurrent(eventDescriptionForm);
						System.out.println("Set event desc as current form");
					}
				}
			}
		});
	}

	public void setFormClosedListener(FormClosedListener closedListener) {
		this.closedListener = closedListener;
	}

	private IEventInstance getSelectedEvent() {
		if (currentlyDisplayedEvents != null) {
			final int selectedEventIndex = getSelectedIndex();
			return selectedEventIndex >= 0 && selectedEventIndex < currentlyDisplayedEvents.length ? currentlyDisplayedEvents[selectedEventIndex]
					: null;
		} else
			return null;
	}

	public void populateEventList(IEventInstance[] events, Date date) {

		currentlyDisplayedEvents = (IEventInstance[]) OldJavaUtils.copyIntoArray(events,
				new IEventInstance[events.length]);

		OldJavaUtils.sort(currentlyDisplayedEvents, new OldJavaUtils.Comparator() {
			public int compare(Object left, Object right) {
				return (int) (((IEventInstance) left).getStart().getTime() - ((IEventInstance) right).getStart()
						.getTime());
			}
		});

		deleteAll();

		setTitle("Events " + PhoneCalendar.dateToString(date));

		System.out.println("displaying " + currentlyDisplayedEvents.length);

		if (currentlyDisplayedEvents.length != 0) {
			for (int i = 0; i < currentlyDisplayedEvents.length; i++) {
				final IEventInstance event = currentlyDisplayedEvents[i];
				append(PhoneCalendar.eventTimesToString(event) + ": " + event.getTitle(), null);
			}
		} else {
			append("No events this day", null);
		}
	}

}
