package phonecalendar;

import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;
import java.util.Vector;

import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.Display;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

import log.ContextAppendingLogger;
import log.FileLogger;
import log.FilteringLogger;
import log.ILogger;
import log.Logger;
import calendar.TimeZonesManager;
import calendar.WrappingException;
import calendar.objectmodel.IEventInstance;
import calendarsourceentries.CalendarSourceEntryStore;

public class PhoneCalendar extends MIDlet {

	private Display display;

	private MainForm mainForm;

	private ILogger log;

	protected void startApp() throws MIDletStateChangeException {
		try {
			log = new ContextAppendingLogger(new FilteringLogger(ILogger.WARNING, new FileLogger(
					"e:/phoneCalendarDebug.txt")));
			Logger.setDefault(log);
		} catch (IOException e1) {
			throw new WrappingException("", e1);
		}

		// log = new ContextAppendingLogger(new OutputStreamLogger(System.out));
		// Logger.setDefault(log);

		final String[] timezones = TimeZone.getAvailableIDs();

		for (int i = 0; i < timezones.length; i++) {
			log.log("timezone " + timezones[i], ILogger.INFO);
		}

		try {
			display = Display.getDisplay(this);

			Vector knownCalendars = null;

			try {
				knownCalendars = CalendarSourceEntryStore.loadCalendarEntries();
			} catch (Exception e) {
				e.printStackTrace();
				log.log(e.toString(), ILogger.ERROR);
				displayAlert(e.getMessage(), display);
			}

			if (knownCalendars == null)
				knownCalendars = new Vector();

			// Select date form
			mainForm = new MainForm(display, log, knownCalendars);
			display.setCurrent(mainForm);
		} catch (Exception e) {
			log.log(e.getMessage(), ILogger.ERROR);
		}
	}

	public static void displayAlert(String alertMessage, Display display) {
		System.out.println("Alert: " + alertMessage);

		final Alert alert = new Alert(alertMessage);
		display.setCurrent(alert);

		// final Form form = new Form("Alert");
		// form.append(alertMessage);
		// display.setCurrent(form);
	}

	static String eventTimesToString(IEventInstance event) {
		return timeToString(event.getStart()) + " - " + timeToString(event.getEnd());
	}

	static String dateToString(Date date) {
		final Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		return integerToTwoCharString(calendar.get(Calendar.DAY_OF_MONTH)) + "/"
				+ integerToTwoCharString(calendar.get(Calendar.MONTH) + 1) + "/" + calendar.get(Calendar.YEAR);
	}

	private static String timeToString(Date date) {
		final Calendar calendar = Calendar.getInstance(TimeZonesManager.EuropeLondon);
		calendar.setTime(date);

		return integerToTwoCharString(calendar.get(Calendar.HOUR_OF_DAY)) + ":"
				+ integerToTwoCharString(calendar.get(Calendar.MINUTE));
		// return date.toString();
	}

	private static String integerToTwoCharString(int number) {
		final String original = Integer.toString(number);
		return original.length() == 1 ? "0" + original : original;
	}

	protected void pauseApp() {
		// TODO Auto-generated method stub

	}

	protected void destroyApp(boolean arg0) throws MIDletStateChangeException {
		final Vector calendars = mainForm.getKnownCalendars();

		log.log("There are " + calendars.size() + " calendars to save", ILogger.DEBUG);

		CalendarSourceEntryStore.saveCalendars(calendars, log);
		log.close();
	}
}
